//
// © 2014 David Vines
//
// Creative Commons License
//
// This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
//

/* jshint sub:true */
/* globals Game: true, vsprintf, alert */

Game.Entity = function(properties) {
    properties = properties || {};
    Game.DynamicGlyph.call(this, properties);
    this._x = properties['x'] || 0;
    this._y = properties['y'] || 0;
    this._z = properties['z'] || 0;
    this._speed = properties['speed'] || 1000;
    this._xp = properties['xp'] || 1;
    this._map = null;
    this._alive = true;
};

Game.Entity.extend(Game.DynamicGlyph);

Game.Entity.prototype.setX = function(x) {
    this._x = x;
};

Game.Entity.prototype.getX = function() {
    return this._x;
};

Game.Entity.prototype.setY = function(y) {
    this._y = y;
};

Game.Entity.prototype.getY = function() {
    return this._y;
};

Game.Entity.prototype.setZ = function(z) {
    this._z = z;
};

Game.Entity.prototype.getZ = function() {
    return this._z;
};

Game.Entity.prototype.setPosition = function(x,y,z) {
    var oldX = this.getX();
    var oldY = this.getY();
    var oldZ = this.getZ();
    this.setX(x);
    this.setY(y);
    this.setZ(z);
    if (this._map) {
        this._map.updateEntityPosition(this, oldX, oldY, oldZ);
    }
};

Game.Entity.prototype.setSpeed = function(speed) {
    this._speed = speed;
};

Game.Entity.prototype.getSpeed = function() {
    return this._speed;
};

Game.Entity.prototype.setMap = function(map) {
    this._map = map;
};

Game.Entity.prototype.getMap = function() {
    return this._map;
};

Game.Entity.prototype.sendMessage = function(recipient, message, args) {
    if (recipient.hasMixin('MessageReader')) {
        if (args) {
            try {
                message = vsprintf(message, args);
            } catch(e) {
                alert(e+" thrown trying vsprinf('"+message+"','"+args+") - line 82");
            }
        }
        recipient.receiveMessage(message);
    }
};

Game.Entity.prototype.sendSelfMessage = function(message,args) {
    if (this.hasMixin('MessageReader')) {
        if (args) {
            try {
                message = vsprintf(message, args);
            } catch(e) {
                alert(e+" thrown trying vsprinf('"+message+"','"+args+") - line 95");
            }
        }
        this.receiveMessage(message);
    }
};

Game.Entity.prototype.sendMessageNearby = function(message, args) {
    if (args) {
        try {
            message = vsprintf(message, args);
        } catch(e) {
            alert(e+" thrown trying vsprinf('"+message+"','"+args+") - line 107");
        }
    }
    var entities = this.getMap().getEntitiesWithinRadius(this.getX(), this.getY(), this.getZ(), 5);
    for(var i=0; i < entities.length; i++) {
        if (entities[i] !== this && entities[i].hasMixin('MessageReader')) {
            entities[i].receiveMessage(message);
        }
    }
};

Game.Entity.prototype.isAlive = function() {
    return this._alive;
};

Game.Entity.prototype.kill = function(message,args) {
    if (!this._alive) {
        return;
    }
    this._alive = false;
    message = message || 'You have died!';
    this.sendSelfMessage(message,args);
    if (!this.hasMixin(Game.EntityMixins.PlayerActor)) {
        this.getMap().removeEntity(this);
    }
};

Game.Entity.prototype.getXP = function() {
    return this._xp;
};

Game.Entity.prototype.switchMap = function(newMap) {
    if (newMap === this.getMap()) {
        return;
    }
    this.getMap().removeEntity(this);
    this._x = 0;
    this._y = 0;
    this._z = 0;
    newMap.addEntity(this);
};